(* $Id: Test_CommandSemantics.ml 7640 2018-09-27 09:17:52Z sutre $ *)


(*
 * Unit tests for the CommandSemantics module.
 *)


open TestCore
open CommandSemantics

(* Import utility functions. *)
open TestUtil.CommandShortcut

(* Helper function to create test messages. *)
let message cmd =
  Format.asprintf "@[<h>[%a]@]" Command.print cmd

let x = var "x"
and y = var "y"
and z = var "z"

let context = Z3.mk_context []

(* Generic test function for the forward translation of program commands. *)
let fwd_aux cmd expected =
  let (f, l) = fwd_formula context cmd
  in
  assert_string
    ~msg:(message cmd)
    expected
    ((Z3.Expr.to_string f) ^ "; [" ^ (String.concat ", " l) ^ "]")

let test_fwd_assign () =
  fwd_aux
    (x := cst 3)
    "(and (= x$ 3)); [x]" ;
  fwd_aux
    (z := z - y)
    "(and (= z$ (- z y))); [z]" ;
  fwd_aux
    (z := x / y)
    "(and (not (= y 0)) (= z$ (div x y))); [z]" ;
  fwd_aux
    (z := x / (y - cst 5))
    "(and (not (= (- y 5) 0)) (= z$ (div x (- y 5)))); [z]" ;
  fwd_aux
    (x := y / (x / z))
    "(and (not (= z 0)) (not (= (div x z) 0)) (= x$ (div y (div x z)))); [x]"

let test_fwd_guard () =
  fwd_aux
    (x != y * z)
    "(and (not (= x (* y z)))); []" ;
  fwd_aux
    (x + cst 5 > y / z)
    "(and (not (= z 0)) (> (+ x 5) (div y z))); []" ;
  fwd_aux
    (x <= y / (cst 3 / z))
    "(and (not (= z 0)) (not (= (div 3 z) 0)) (<= x (div y (div 3 z)))); []" ;
  fwd_aux
    (x / y == cst 1 / z)
    "(and (not (= y 0)) (not (= z 0)) (= (div x y) (div 1 z))); []"

let test_fwd_skip () =
  fwd_aux (skip) "true; []"

(* Generic test function for the backward translation of program commands. *)
let bwd_aux cmd expected =
  let (f, l) = bwd_formula context cmd
  in
  assert_string
    ~msg:(message cmd)
    expected
    ((Z3.Expr.to_string f) ^ "; [" ^ (String.concat ", " l) ^ "]")

let test_bwd_assign () =
  bwd_aux
    (x := cst 3)
    "(and (= x 3)); [x]" ;
  bwd_aux
    (z := z - y)
    "(and (= z (- z$ y))); [z]" ;
  bwd_aux
    (z := x / y)
    "(and (not (= y 0)) (= z (div x y))); [z]" ;
  bwd_aux
    (z := x / (y - cst 5))
    "(and (not (= (- y 5) 0)) (= z (div x (- y 5)))); [z]" ;
  bwd_aux
    (x := y / (x / z))
    "(and (not (= z 0)) (not (= (div x$ z) 0)) (= x (div y (div x$ z)))); [x]"

let test_bwd_guard () =
  bwd_aux
    (x != y * z)
    "(and (not (= x (* y z)))); []" ;
  bwd_aux
    (x + cst 5 > y / z)
    "(and (not (= z 0)) (> (+ x 5) (div y z))); []" ;
  bwd_aux
    (x <= y / (cst 3 / z))
    "(and (not (= z 0)) (not (= (div 3 z) 0)) (<= x (div y (div 3 z)))); []" ;
  bwd_aux
    (x / y == cst 1 / z)
    "(and (not (= y 0)) (not (= z 0)) (= (div x y) (div 1 z))); []"

let test_bwd_skip () =
  bwd_aux (skip) "true; []"

(* Collection of all tests. *)
let alltests =
  [
    "cmdsemantics.fwd_assign", test_fwd_assign ;
    "cmdsemantics.fwd_guard", test_fwd_guard ;
    "cmdsemantics.fwd_skip", test_fwd_skip ;
    "cmdsemantics.bwd_assign", test_bwd_assign ;
    "cmdsemantics.bwd_guard", test_bwd_guard ;
    "cmdsemantics.bwd_skip", test_bwd_skip ;
  ]

(* This test suite. *)
let suite = ("CommandSemantics", alltests)
